package shark.arthas.demo;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @author yuwc
 * @ClassName 线程被锁定或死锁
 * @description: 线程被锁定或死锁
 * @date 2024年03月18日
 * @version: 1.0
 */
@RestController
@RequestMapping("01")
public class Sample01 {
    private ReentrantLock lock1 = new ReentrantLock();
    private ReentrantLock lock2 = new ReentrantLock();

    @GetMapping("block")
    public void block() throws InterruptedException {
        Thread th1 = new Thread(this::doBlock, "thread1");
        th1.start();
        TimeUnit.MILLISECONDS.sleep(100);
        Thread th2 = new Thread(this::doBlock, "thread2");
        th2.start();
    }

    @GetMapping("deadLock")
    public void deadBlock() throws InterruptedException {
        Thread th1 = new Thread(this::doDeadLock, "thread1");
        th1.start();
//        TimeUnit.MILLISECONDS.sleep(50);
        Thread th2 = new Thread(this::doDeadLock, "thread2");
        th2.start();
    }

    private synchronized void doBlock() {
        try {
            TimeUnit.MINUTES.sleep(1);
            System.out.println("method running...");
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    private void doDeadLock() {
        try {
            String threadName = Thread.currentThread().getName();
            if ("thread1".equalsIgnoreCase(threadName)) {
                if (lock1.tryLock(1, TimeUnit.MINUTES)) {
                    TimeUnit.MILLISECONDS.sleep(100);
//                    lock2.lock();
                    if (lock2.tryLock(1, TimeUnit.MINUTES)) {
                        TimeUnit.SECONDS.sleep(30);
                        System.out.println("method running...");
                    }
                }
            }else {
                if (lock2.tryLock(1, TimeUnit.MINUTES)) {
                    TimeUnit.MILLISECONDS.sleep(100);
//                    lock1.lock();
                    if (lock1.tryLock(1, TimeUnit.MINUTES)) {
                        TimeUnit.SECONDS.sleep(30);
                        System.out.println("method running...");
                    }
                }
            }
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        } finally {
            if (lock1.isLocked()) {
                lock1.unlock();
            }
            if (lock2.isLocked()) {
                lock2.unlock();
            }
        }
    }
}
